package ostate;

import ostate.util.properties.Properties;
import ostate.invoke.EventProcessor;

/**
 * <p>
 * The interface of state machine.
 * </p><p>
 * State machine clients use it
 * to initiate state machine, using
 * <code>sm.changeState(...)</code> call,
 * then they use it to sent events, using <code>processEvent</code>
 * method.
 * </p><p>
 * States use it to perform transitions, using
 * <code>sm.changeState(...)</code> call.
 * </p>
 *
 * @author Pavel Mavrin
 * @version $Id$
 */
public interface IStateMachine<I> {

    void changeState(Class<? extends IState<I>> newStateClass, Properties properties, Runnable action);

    void changeState(Class<? extends IState<I>> newStateClass, Properties properties);

    void changeState(Class<? extends IState<I>> newStateClass);

    void processEvent(Event event, EventProcessor eventProcessor);

    void processEvent(Event event);

    <T extends IState<I>> T getCurrentStateByClass(Class<T> stateClass);
}