package ostate;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Proxy;

/**
 * Factory that produces State Machine instances.
 * Objects, produced by this factory, implement
 * two interface: I and {@link IStateMachine IStateMachine&lt;I&gt}.
 *
 * @author Pavel Mavrin
 * @version $Id$
 */
public class StateMachineFactory<I> {

    private final static StateMachineFactory ourInstance =
            new StateMachineFactory();

    private StateMachineFactory() {
    }

    public static <T> StateMachineFactory<T> newInstance() {
        return ourInstance;
    }

    public IStateMachine<I> createStateMachine(Class<I> AI,
                                              IStateFactory<I> sf) {
        InvocationHandler handler = new StateMachineManager<I>(AI, sf);
        IStateMachine<I> proxy = (IStateMachine<I>)Proxy.newProxyInstance(AI.getClassLoader(), new Class[] {AI, IStateMachine.class}, handler);
        sf.setStateMachine(proxy);
        return proxy;
    }

}