package ostate.discriptions;

import ostate.Event;

import java.lang.reflect.Method;

/**
 * Wrapper for event handler method.
 *
 * @author Pavel Mavrin
 * @version $Id$
 */

public class Handler implements Comparable<Handler> {

    private final Class<? extends Event> event;

    private final Method handler;

    /**
     * Default constructor.
     * @param eventClass class of events, handled by handler.
     * @param handler handler method.
     */
    public Handler(Class<? extends Event> eventClass, Method handler) {
        this.event = eventClass;
        this.handler = handler;
    }

    /**
     * Returns class of events, handled by this handler.
     * @return class of events, handled by this handler.
     */
    public Class<? extends Event> getEvent() {
        return event;
    }

    /**
     * Returns handler method.
     * @return handler method.
     */
    public Method getHandler() {
        return handler;
    }

    /**
     * Handlers will be sorted from superclasses to subclasses.
     * @param o handler to compare to.
     * @return 1 if our handler must be invoked after o, -1 if before, 0 if it does not matter.
     */
    public int compareTo(Handler o) {
        if (!handler.getDeclaringClass().isAssignableFrom(
                o.getHandler().getDeclaringClass())) return 1;
        if (!o.getHandler().getDeclaringClass().isAssignableFrom(
                handler.getDeclaringClass())) return -1;
        return 0;
    }
}
