package ostate.exceptions;

/**
 * State class, trying to be active state is invalid state class.
 * Valid state class for {@link ostate.IStateMachine IStateMachine&lt;I&gt;} must:
 * <list>
 * <li>implement I</li>
 * <li>implement {@link ostate.IState IState&lt;I&gt;}</li>
 * <li>have {@link ostate.annotations.State} annotation</li>
 * </list>
 *
 * @author Pavel Mavrin
 * @version $Id$
 */
public class InvalidStateException extends RuntimeException {

    protected final Class stateClass;

    protected final Throwable cause;

    public InvalidStateException(String message, Class stateClass, Throwable cause) {
        super(message + ": " + stateClass);
        this.stateClass = stateClass;
        this.cause = cause;
    }

    public InvalidStateException(String message, Class stateClass) {
        this(message, stateClass, null);
    }

    public Class getStateClass() {
        return stateClass;
    }

    public Throwable getCause() {
        return cause;
    }
}
