package ostate.invoke.impl;

import ostate.util.tree.Node;
import ostate.discriptions.StateDiscriptorFactory;
import ostate.discriptions.StateDiscriptor;
import ostate.discriptions.Handler;
import ostate.invoke.EventProcessingContext;
import ostate.invoke.EventProcessor;
import ostate.IState;
import ostate.Event;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Stack;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;

/**
 * EventProcessingContextImpl to be documented.
 *
 * @author Pavel Mavrin
 * @version $Id$
 */

public class EventProcessingContextImpl<I> implements EventProcessingContext {

    private final Node<IState<I>> node;
    private final Event event;
    private final EventProcessor processor;
    private final Iterator<Node<IState<I>>> childrenIterator;
    private final Stack<IState<I>> sources;

    public EventProcessingContextImpl(Node<IState<I>> node, Event event, EventProcessor processor, Stack<IState<I>> sources) {
        this.node = node;
        this.event = event;
        this.processor = processor;
        this.sources = sources;
        childrenIterator = node.getChildren().iterator();
    }


    public void processThisState() throws IllegalAccessException, InvocationTargetException {
        sources.push(node.getValue());
        try {
            StateDiscriptor<I> sd = StateDiscriptorFactory.getStateDiscriptor(node.getValue());
            for (Handler handler : sd.getHandlers()) {
                if (handler.getEvent().isInstance(event)) {
                    handler.getHandler().invoke(node.getValue(), event);
                }
            }
        } finally {
            sources.pop();
        }
    }

    public void processNextChild() throws IllegalAccessException, InvocationTargetException {
        processor.processEvent(new EventProcessingContextImpl<I>(
                childrenIterator.next(),
                event,
                processor,
                sources));
    }

    public boolean hasMoreChildren() {
        return childrenIterator.hasNext();
    }

}
