package ostate.util.properties;

import java.util.Map;
import java.util.HashMap;

/**
 * DefaultProperty to be documented.
 *
 * @author Pavel Mavrin
 * @version $Id$
 */

public class DefaultProperty<V> implements Property<V> {

    private final String name;

    private static final Map<String, Property> created = new HashMap<String, Property>();

    private DefaultProperty(String name) {
        this.name = name;
    }

    public String getName() {
        return name;
    }

    public static <T> Property<T> createProperty(String name) {
        synchronized (created) {
            Property<T> result = created.get(name);
            if (result == null) {
                result = new DefaultProperty<T>(name);
                created.put(name, result);
            }
            return result;
        }
    }

}
