package ostate.util.tree;

import ostate.IState;
import ostate.util.tree.Node;

import java.util.HashMap;
import java.util.Map;

/**
 * StatesTree to be documented.
 *
 * @author Pavel Mavrin
 * @version $Id$
 */
public class StatesTree<I>
        extends Tree<IState<I>>
        implements Cloneable {

    private final Map<Class, IState<I>> statesMap = new HashMap<Class, IState<I>>();

    public StatesTree() {
    }

    public StatesTree(IState<I> rootValue) {
        super(rootValue);
    }

    public void registerNode(Node<IState<I>> node) {
        super.registerNode(node);
        if (statesMap == null) {
            //statesMap = new HashMap<Class, AState<I, E>>();
        }
        statesMap.put(
                node.getValue().getClass(),
                node.getValue()
        );
    }

    public void disregisterNode(Node<IState<I>> node) {
        super.disregisterNode(node);
        statesMap.remove(
                node.getValue().getClass()
        );
    }

    public Node<IState<I>> getNode(Class nodeClass) {
        if (statesMap == null) {
            return null;
        } else {
            return getNode(statesMap.get(nodeClass));
        }
    }

    public StatesTree<I> clone() {
        StatesTree<I> clone = new StatesTree<I>();
        if (getRoot() != null) {
            copyBranchToTree(getRoot(), clone, null);
        }
        return clone;
    }

    private Node<IState<I>> copyBranchToTree(Node<IState<I>> node, StatesTree<I> tree, Node<IState<I>> parent) {
        Node<IState<I>> copy;
        if (parent == null) {
            tree.setRoot(node.getValue());
            copy = tree.getRoot();
        } else {
            copy = new Node<IState<I>>(tree, parent, node.getValue());
            parent.addChild(copy);
        }
        for (Node<IState<I>> child : node.getChildren()) {
            copyBranchToTree(child, tree, copy);
        }
        return copy;
    }
}
