package ostate.util.tree;

import ostate.util.tree.ModifyingLockedTreeException;
import ostate.util.tree.Node;

import java.util.Map;
import java.util.HashMap;

/**
 * Tree to be documented.
 *
 * @author Pavel Mavrin
 * @version $Id$
 */
public class Tree<V> {

    private Node<V> root;

    private final Map<V, Node<V>> nodeMap = new HashMap<V, Node<V>>();

    private boolean locked = false;

    public Tree() {
    }

    public Tree(V rootValue) {
        this.root = new Node<V>(this, null, rootValue);
    }

    public void registerNode(Node<V> node) {
        if (isLocked()) {
            throw new ModifyingLockedTreeException();
        }
        nodeMap.put(node.getValue(), node);
    }

    public void disregisterNode(Node<V> node) {
        if (isLocked()) {
            throw new ModifyingLockedTreeException();
        }
        nodeMap.remove(node.getValue());
    }

    public Node<V> getNode(V value) {
        return nodeMap.get(value);
    }

    public void setRoot(V rootValue) {

        if (root != null) {
            Node<V>[] children = new Node[root.getChildren().size()];
            children = root.getChildren().toArray(children);

            for (Node<V> node : children) {
                root.removeChild(node);
            }
            disregisterNode(root);
        }
        root = new Node<V>(this, null, rootValue);
    }

    public Node<V> getRoot() {
        return root;
    }

    public void setLocked(boolean locked) {
        this.locked = locked;
    }

    public boolean isLocked() {
        return locked;
    }
}
